from tcsoa.gen.Internal_Search._2020_12.services import SearchFolderService as imp0
from tcsoa.base import TcService


class SearchFolderService(TcService):
    createOrEditSearchFolders = imp0.createOrEditSearchFolders
    exportSearchFolders = imp0.exportSearchFolders
    getSearchFolderAccessors = imp0.getSearchFolderAccessors
    getTranslatedReportSearchRecipe = imp0.getTranslatedReportSearchRecipe
    importSearchFolder = imp0.importSearchFolder
