from tcsoa.gen.ActiveWorkspaceVis._2015_03.services import DataManagementService as imp0
from tcsoa.base import TcService


class DataManagementService(TcService):
    createLaunchInfo = imp0.createLaunchInfo
